'''
    Mapseg, a program for segmenting maps and producing OSM polygons
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''
from shapely.geometry import Polygon

def ConvertPolyToShapely(polys):
	#Convert list of shapely polygons to dict for easier manipulation
	polysDict = {}
	for poly in polys:
		tempPoly = {}
		tempPoly['path'] = Polygon(poly['path'])
		tempPoly['meta'] = poly['meta']
		if tempPoly['path']._geom==None: continue #Remove null geometries
		polysDict[id(poly)] = tempPoly
	return polysDict

def GroupIntersecting(polysDict):


	#Group polygons that intersect
	groups = []
	while len(polysDict) > 0:
		firstId = polysDict.keys()[0]
		group = []
		firstPoly = polysDict[firstId]
		group.append(firstPoly)
		del polysDict[firstId]
		polysInters = []
		for polyId in polysDict.keys():
			try:
				if group[0]['path'].intersects(polysDict[polyId]['path']):
					group.append(polysDict[polyId])
					del polysDict[polyId]
			except ValueError:
				print group[0], polysDict[polyId]
				exit(0)
		groups.append(group)
	return groups

def FindOuterPolys(groups):
	#For each group, find which is outer polygon
	finalPolys = []
	for group in groups:
		maxArea = None
		maxPoly = None
		maxPolyId = None
		for poly,i in zip(group,range(len(group))):
			ar = poly['path'].area
			#print id(poly), ar
			if maxArea == None or ar > maxArea:
				maxArea = ar
				maxPoly = poly
				maxPolyId = i
		outer = maxPoly
		inner = group
		del inner[maxPolyId]
		#print "outer",outer
		#print "inner",inner
	
		out = {}
		out['outer'] = outer
		out['inner'] = []
		for innerPoly in inner:
			out['inner'].append(innerPoly)
		finalPolys.append(out)
	return finalPolys

def ConvertShapelyToPoly(shapelyPolys):
	out = []
	for poly in shapelyPolys:
		group = {}
		group['outer'] = {}
		group['outer']['path'] = list(poly['outer']['path'].exterior.coords)
		group['outer']['meta'] = poly['outer']['meta']
		group['inner'] = []
		for inner in poly['inner']:
			tempPoly = {}
			tempPoly['path'] = list(inner['path'].exterior.coords)
			tempPoly['meta'] = inner['meta']
			group['inner'].append(tempPoly)
		out.append(group)
	return out
		

