'''
    Mapseg, a program for segmenting maps and producing OSM polygons
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from OSGB import *
from OSTN02 import *

def PixToLatLon(pos,imsize,bounds):
	#print pos,imsize
	px = pos[0]/imsize[0]
	py = pos[1]/imsize[1]
	lon = bounds[1] + px * (bounds[3] - bounds[1])
	lat = bounds[0] + py * (bounds[2] - bounds[0])
	return (lat, lon)

def TransformPolysToWorld(polys, imsize, bounds):
	polysOut = []
	for poly in polys:
		polyOut = {}
		outer = poly['outer']
		polyOut['outer'] = {}
		polyOut['outer']['path'] = []
		polyOut['outer']['meta'] = poly['outer']['meta']
		for point in outer['path']:
			worldPos = PixToLatLon(point,imsize,bounds)

			polyOut['outer']['path'].append(worldPos)
		
		polyOut['inner'] = []
		for inPoly in poly['inner']:
			inWorld = {}
			inWorld['path'] = []
			inWorld['meta'] = inPoly['meta']
			for point in inPoly['path']:
				#print "inner",point
				worldPos = PixToLatLon(point,imsize,bounds)
				inWorld['path'].append(worldPos)
			polyOut['inner'].append(inWorld)
		polysOut.append(polyOut)
	return polysOut

def TransformOsgbToWgs84(polys):
	transPolys = []
	for poly in polys:
		transPoly = {}
		transPoly['outer'] = {}
		transPoly['outer']['path'] = []
		outer = poly['outer']
		transPoly['outer']['meta'] = outer['meta']
		for point in outer['path']:
			(x, y, z) = OSGB36_to_ETRS89(point[1], point[0], 0.0)
			(gla, glo) = grid_to_ll(x, y, 'WGS84')
			transPoly['outer']['path'].append((gla,glo))

		if len(poly['inner']) > 0:
			transPoly['outer']['meta']['mapseg:inner-polygon']= 'FIXME'

		transPolys.append(transPoly)
	return transPolys


