'''
    Mapseg, a program for segmenting maps and producing OSM polygons
    Copyright (C) 2010 Tim Sheerman-Chase

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 or version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

from PIL import Image
import numpy as np


def Normalise(im):
	out = Image.new("L",im.size)
	outpix = out.load()
	testpix = im.load()
	maxVal = None
	for i in range(im.size[0]):
		for j in range(im.size[1]):
			#print testpix[i,j]
			if maxVal == None or maxVal < testpix[i,j]:
				maxVal = testpix[i,j]

	for i in range(im.size[0]):
		for j in range(im.size[1]):
			outpix[i,j] = testpix[i,j] * 255.0 / maxVal

	return out

def DistSelectColour(im, col, threshold=50.0):
	pix = im.load()
	selection = Image.new("F",im.size)
	sel = selection.load()
	colAr = np.array(col)
	if len(pix[0,0])==0:
		print "Only one colour channel. Is the image index coloured?"
	progressPrinted = -1.0
	count = 0
	numPix = im.size[0] * im.size[1]
	for i in range(im.size[0]):
		for j in range(im.size[1]):
			progress = float(count) / float(numPix)
			if progress - progressPrinted > 0.1:
				progressPrinted = progress
				print progress * 100,"%"

			total = 0.0
			for d,c in zip(pix[i,j],col):
				diff = d - c
				total = total + diff * diff
			total = total ** 0.5
			#dist = ((np.array(pix[i,j]) - colAr)**2.0).sum()**0.5
			#print pix[i,j], colAr
			if(total > threshold):
				total = threshold

			sel[i,j] = total

			count = count + 1
	return selection

def ThresholdImageBelowT(im, threshold=50.0):
	pix = im.load()
	selection = Image.new("1",im.size)
	sel = selection.load()

	for i in range(im.size[0]):
		for j in range(im.size[1]):
			if pix[i,j] < threshold: sel[i,j] = 1
			else: sel[i,j] = 0
	
	return selection

def SelectionToEdge(im):
	#For each selected pixel, check if on an edge
	pix = im.load()
	edges = {}
	for i in range(im.size[0]):
		for j in range(im.size[1]):
			if pix[i,j] != 0:
				#Check surrounding pixels for non-selected			
				def CheckEdge(a,b,c,d,e,f):
					addEdge = 0
					try:
						if (pix[i+a,j+b] == 0):
							addEdge = 1
					except IndexError:
						addEdge = 1
					if addEdge:
						if (i+c,j+d) not in edges:
							edges[(i+c,j+d)] = {}
						edges[(i+c,j+d)][(a,b)] = (i+e,j+f)

				CheckEdge(-1,0,0,1,0,0)
				CheckEdge(+1,0,1,0,1,1)
				CheckEdge(0,-1,0,0,1,0)
				CheckEdge(0,+1,1,1,0,1)
	return edges

def EdgeFragmentsToPolygons(edges, tilesize, roi=[]): #roi is of the form [xmin, xmax, ymin, ymax] in pixels

	startingPositions = {}
	for cursorPos in edges.keys():
		if len(roi) > 0: #Filter starting positions based on region of interest
			if cursorPos[0] < roi[0] or cursorPos[0] > roi[1]: continue
			if cursorPos[1] < roi[2] or cursorPos[1] > roi[3]: continue
		startingPositions[cursorPos] = {}
		edgesAtPos = edges[cursorPos]
		startingPositions[cursorPos] = edgesAtPos.copy()

	#print edges
	paths = []
	while len(startingPositions) > 0:
		startKey = startingPositions.keys()[0]
		startPos = startingPositions.values()[0].values()[0]
		#print edges.values()
		#exit(0)
		cursor = startPos
		count = 0
		path = {}
		path['path'] = []
		path['meta'] = {}

		brokenPoly = 0
		while (cursor != startPos or len(path['path']) == 0) and not brokenPoly:
			#print "cursor",cursor
			if cursor not in edges:
				print "cursor break at", cursor
				del edges[startKey]
				del startingPositions[startKey]
				brokenPoly = 1
				continue
			if cursor[0] <= 0 or cursor[1] <= 0 or cursor[0] >= tilesize[0]-1 or cursor[1] >= tilesize[1]-1:
				path['meta']['mapseg:tile-edge'] = 'FIXME'

			edgeId = edges[cursor].keys()[0]
			edgeEnd = edges[cursor][edgeId]
			path['path'].append(cursor)
			del edges[cursor][edgeId]
			if len(edges[cursor]) == 0: del edges[cursor]
			if cursor in startingPositions:
				del startingPositions[cursor][edgeId]
				if len(startingPositions[cursor]) == 0: del startingPositions[cursor]
			cursor = edgeEnd

		if not brokenPoly:
			paths.append(path)


	return paths



